/////////////////////////////////////////////////////////
// Markisensteuerung
// mit Arduino und ESP8266 (mit Arduino IDE Integration)
// Teil 1: Webserver Code für ESP8266
// Stefan Thesen 05/2015
//
// Free for anybody - no warranties
/////////////////////////////////////////////////////////

#include <ESP8266WiFi.h>

// WLAN where we want to be client - put your SSID/password here
const char* client_ssid = "YOUR-SSID-HERE";
const char* client_pwd = "YOUR-WLAN-PASSWORD-HERE";

// counters
unsigned long ulReqcount;
unsigned long ulReconncount;

// Create an instance of the server
// specify the port to listen on as an argument
WiFiServer server(80);


////////////////
// setup routine
////////////////
void setup() 
{
  ulReqcount=0; 
  ulReconncount=0;
  
  // make sure this is identical to the Arduino speed
  Serial.begin(9600);
  
  // inital connect
  WiFi.hostname("markise");
  WiFi.mode(WIFI_STA);
  WiFiStart();
}


//////////////////
// Connect to WiFi
//////////////////
void WiFiStart()
{
  ulReconncount++;
  
  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(client_ssid);
  
  WiFi.begin(client_ssid, client_pwd);
  
  while (WiFi.status() != WL_CONNECTED) 
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  
  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());
}


//////////////
//loop routine
//////////////
void loop() 
{
  // check if WLAN is connected
  if (WiFi.status() != WL_CONNECTED)
  {
    WiFiStart();
  }
  
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) 
  {
    return;
  }
  
  // Wait until the client sends some data
  Serial.println("new client");
  unsigned long ultimeout = millis()+250;
  while(!client.available() && (millis()<ultimeout) )
  {
    delay(1);
  }
  if(millis()>ultimeout) 
  { 
    Serial.println("client connection time-out!");
    return; 
  }
  
  // Read the first line of the request
  String sRequest = client.readStringUntil('\r');
  client.flush();
  
  if(sRequest=="")
  {
    Serial.println("empty request! - stopping client");
    client.stop();
    return;
  }
  
  // get path; end of path is either space or ?
  // Syntax is e.g. GET /?pin=MOTOR1STOP HTTP/1.1
  String sPath="",sParam="", sCmd="";
  String sGetstart="GET ";
  int iStart,iEndSpace,iEndQuest;
  iStart = sRequest.indexOf(sGetstart);
  if (iStart>=0)
  {
    iStart+=+sGetstart.length();
    iEndSpace = sRequest.indexOf(" ",iStart);
    iEndQuest = sRequest.indexOf("?",iStart);
    
    // are there parameters?
    if(iEndSpace>0)
    {
      if(iEndQuest>0)
      {
        // there are parameters
        sPath  = sRequest.substring(iStart,iEndQuest);
        sParam = sRequest.substring(iEndQuest,iEndSpace);
      }
      else
      {
        // NO parameters
        sPath  = sRequest.substring(iStart,iEndSpace);
      }
    }
    
    /*Serial.print("Path=");
    Serial.println(sPath);
    Serial.print("Parameters=");
    Serial.println(sParam);*/
  }
  
  /////////////////////////////////////////////////////////////////
  // output command to serial so that we can read it on the Arduino
  /////////////////////////////////////////////////////////////////
  if(sParam.length()>0)
  {
    int iEqu=sParam.indexOf("=");
    if(iEqu>=0)
    {
      sCmd = sParam.substring(iEqu+1,sParam.length());
      
      // output to serial, which is connceted to the Arduino
      Serial.println(sCmd);
    }
  }
  
  ///////////////////////////
  // format the html response
  ///////////////////////////
  String sResponse,sHeader;
  
  ////////////////////////////
  // 404 for non-matching path
  ////////////////////////////
  if(sPath!="/")
  {
    sResponse="<html><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested URL was not found on this server.</p></body></html>";
    
    sHeader  = "HTTP/1.1 404 Not found\r\n";
    sHeader += "Content-Length: ";
    sHeader += sResponse.length();
    sHeader += "\r\n";
    sHeader += "Content-Type: text/html\r\n";
    sHeader += "Connection: close\r\n";
    sHeader += "\r\n";
  }
  ///////////////////////
  // format the html page
  ///////////////////////
  else
  {
    ulReqcount++;
    sResponse  = "<html><head><title>Markisensteuerung</title></head><body>";
    sResponse += "<font color=\"#000000\"><body bgcolor=\"#d0d0f0\">";
    sResponse += "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=yes\">";
    sResponse += "<h1>Markisensteuerung</h1>";
    sResponse += "<FONT SIZE=+1>";
    sResponse += "<p>Hauptmarkise <a href=\"?pin=MOTOR1UP\"><button>einfahren</button></a>&nbsp;<a href=\"?pin=MOTOR1STOP\"><button>stop</button></a>&nbsp;<a href=\"?pin=MOTOR1DOWN\"><button>ausfahren</button></a></p>";
    sResponse += "<p>Seitenmarkise <a href=\"?pin=MOTOR2UP\"><button>einfahren</button></a>&nbsp;<a href=\"?pin=MOTOR2STOP\"><button>stop</button></a>&nbsp;<a href=\"?pin=MOTOR2DOWN\"><button>ausfahren</button></a></p>";
    
    if (sCmd.length()>0)
    {
      sResponse += "Kommando:" + sCmd + "<BR>";
    }
    sResponse += "<FONT SIZE=-2>";
    sResponse += "<BR>Aufrufz&auml;hler="; 
    sResponse += ulReqcount;
    sResponse += " - Verbindungsz&auml;hler="; 
    sResponse += ulReconncount;
    sResponse += "<BR>";
    sResponse += "Stefan Thesen 03/2015<BR>";
    sResponse += "</body></html>";
    
    sHeader  = "HTTP/1.1 200 OK\r\n";
    sHeader += "Content-Length: ";
    sHeader += sResponse.length();
    sHeader += "\r\n";
    sHeader += "Content-Type: text/html\r\n";
    sHeader += "Connection: close\r\n";
    sHeader += "\r\n";
  }
  
  // Send the response to the client
  client.print(sHeader);
  client.print(sResponse);
  client.stop();

  Serial.println("Client disonnected");
}

